@@ -68,7 +68,7 @@ static char _default_uuid_prefix[DM_MAX_UUID_PREFIX_LEN + 1] = "LVM-";
 
 static int _verbose = 0;
 static int _suspended_dev_counter = 0;
-static int _name_mangling_mode = -1;
+static dm_string_mangling_t _name_mangling_mode = DEFAULT_DM_NAME_MANGLING;
 
 #ifdef HAVE_SELINUX_LABEL_H
 static struct selabel_handle *_selabel_handle = NULL;
@@ -526,7 +526,7 @@ static int _dm_task_set_name_from_path(struct dm_task *dmt, const char *path,
 		return 0;
 	}
 
-	if (!stat(path, &st2) && (st1.st_rdev == st2.st_rdev))
+	if (!stat(buf, &st2) && (st1.st_rdev == st2.st_rdev))
 		final_name = name;
 	else if (_find_dm_name_of_device(st1.st_rdev, buf, sizeof(buf)))
 		final_name = buf;
