@@ -70,6 +70,7 @@ prepare_lvmetad() {
 	}
 
 	lvmconf "global/use_lvmetad = 1"
+	lvmconf "devices/md_component_detection = 0"
 
 	lvmetad -f "$@" -s $TESTDIR/lvmetad.socket &
 	echo "$!" > LOCAL_LVMETAD
@@ -77,6 +78,12 @@ prepare_lvmetad() {
 	sleep 1
 }
 
+notify_lvmetad() {
+	if test -e LOCAL_LVMETAD; then
+		pvscan --cache "$@" || true
+	fi
+}
+
 teardown_devs() {
 	# Delete any remaining dm/udev semaphores
 	teardown_udev_cookies
@@ -323,8 +330,11 @@ disable_dev() {
 
 	init_udev_transaction
 	for dev in "$@"; do
-        	dmsetup remove -f $dev || true
-		pvscan --lvmetad $dev || true
+	    maj=$(($(stat --printf=0x%t $dev)))
+	    min=$(($(stat --printf=0x%T $dev)))
+	    echo "disabling device $dev ($maj:$min)"
+            dmsetup remove -f $dev || true
+	    notify_lvmetad $maj:$min
 	done
 	finish_udev_transaction
 
@@ -338,7 +348,7 @@ enable_dev() {
 		dmsetup create -u TEST-$name $name $name.table || dmsetup load $name $name.table
 		# using device name (since device path does not exists yes with udev)
 		dmsetup resume $name
-		pvscan --lvmetad $dev || true
+		notify_lvmetad $dev
 	done
 	finish_udev_transaction
 }
