@@ -380,7 +380,9 @@ static struct dm_split_name *_get_split_name(const char *uuid, const char *name,
 		return NULL;
 	}
 
-	split_name->subsystem = _extract_uuid_prefix(uuid, separator);
+	if (!(split_name->subsystem = _extract_uuid_prefix(uuid, separator)))
+		return_NULL;
+
 	split_name->vg_name = split_name->lv_name =
 	    split_name->lv_layer = (char *) "";
 
@@ -1867,6 +1869,9 @@ static int _display_name(CMD_ARGS)
 {
 	char dev_name[PATH_MAX];
 
+	if (!names)
+		return 1;
+
 	if ((_dev_name_type == DN_BLK || _dev_name_type == DN_MAP) &&
 	    dm_device_get_name((int) MAJOR(names->dev), (int) MINOR(names->dev),
 			       _dev_name_type == DN_BLK, dev_name, PATH_MAX))
@@ -2201,7 +2206,8 @@ static void _display_tree_walk_children(struct dm_tree_node *node,
 
 static int _add_dep(CMD_ARGS)
 {
-	if (!dm_tree_add_dev(_dtree, (unsigned) MAJOR(names->dev), (unsigned) MINOR(names->dev)))
+	if (names &&
+	    !dm_tree_add_dev(_dtree, (unsigned) MAJOR(names->dev), (unsigned) MINOR(names->dev)))
 		return 0;
 
 	return 1;
@@ -3240,7 +3246,8 @@ static int _loop_table(char *table, size_t tlen, char *file,
 	blksize = fsbuf.f_frsize;
 #endif
 
-	close(fd);
+	if (close(fd))
+                log_sys_error("close", file);
 
 	if (dm_snprintf(table, tlen, "%llu %llu loop %s %llu\n", 0ULL,
 			(long long unsigned)sectors, file, (long long unsigned)off) < 0)
@@ -3252,8 +3259,9 @@ static int _loop_table(char *table, size_t tlen, char *file,
 	return 1;
 
 error:
-	if (fd > -1)
-		close(fd);
+	if (fd > -1 && close(fd))
+		log_sys_error("close", file);
+
 	return 0;
 }
 
@@ -3413,7 +3421,8 @@ static int _process_options(const char *options)
 
 static int _process_switches(int *argc, char ***argv, const char *dev_dir)
 {
-	char *base, *namebase, *s;
+	const char *base;
+	char *namebase, *s;
 	static int ind;
 	int c, r;
 
@@ -3478,7 +3487,7 @@ static int _process_switches(int *argc, char ***argv, const char *dev_dir)
 		fprintf(stderr, "Failed to duplicate name.\n");
 		return 0;
 	}
-	base = basename(namebase);
+	base = dm_basename(namebase);
 
 	if (!strcmp(base, "devmap_name")) {
 		free(namebase);
@@ -3612,7 +3621,7 @@ static int _process_switches(int *argc, char ***argv, const char *dev_dir)
 				log_error("Unknown name mangling mode");
 				return 0;
 			}
-			dm_set_name_mangling_mode(_int_args[MANGLENAME_ARG]);
+			dm_set_name_mangling_mode((dm_string_mangling_t) _int_args[MANGLENAME_ARG]);
 		}
 		if (ind == NAMEPREFIXES_ARG)
 			_switches[NAMEPREFIXES_ARG]++;
