@@ -93,8 +93,9 @@ static int _lvcreate_name_params(struct lvcreate_params *lp,
 				lp->origin = ptr + 1;
 		}
 
-		if (!lp->vg_name)
-			_set_vg_name(lp, extract_vgname(cmd, NULL));
+		if (!lp->vg_name &&
+		    !_set_vg_name(lp, extract_vgname(cmd, NULL)))
+			return_0;
 
 		if (!lp->vg_name) {
 			log_error("The origin name should include the "
@@ -115,8 +116,9 @@ static int _lvcreate_name_params(struct lvcreate_params *lp,
 			if (!_set_vg_name(lp, extract_vgname(cmd, lp->pool)))
 				return_0;
 
-			if (!lp->vg_name)
-				_set_vg_name(lp, extract_vgname(cmd, NULL));
+			if (!lp->vg_name &&
+			    !_set_vg_name(lp, extract_vgname(cmd, NULL)))
+				return_0;
 
 			if (!lp->vg_name) {
 				log_error("The pool name should include the "
@@ -291,15 +293,29 @@ static int _update_extents_params(struct volume_group *vg,
 			break;
 	}
 
-	if (lp->create_thin_pool && !lp->poolmetadatasize)
-		/* Defaults to nr_pool_blocks * 64b */
-		lp->poolmetadatasize =  (uint64_t) lp->extents * vg->extent_size /
-			(uint64_t) lp->chunk_size * UINT64_C(64);
+	if (lp->create_thin_pool) {
+		if (!arg_count(vg->cmd, poolmetadatasize_ARG))
+			/* Defaults to nr_pool_blocks * 64b */
+			lp->poolmetadatasize =  (uint64_t) lp->extents * vg->extent_size /
+				(uint64_t) (lp->chunk_size * (SECTOR_SIZE / UINT64_C(64)));
 
-	if (lp->poolmetadatasize &&
-	    !(lp->poolmetadataextents = extents_from_size(vg->cmd, lp->poolmetadatasize,
-							  vg->extent_size)))
-		return_0;
+		if (lp->poolmetadatasize > (2 * DEFAULT_THIN_POOL_MAX_METADATA_SIZE)) {
+			if (arg_count(vg->cmd, poolmetadatasize_ARG))
+				log_warn("WARNING: Maximum supported pool metadata size is 16GB.");
+			lp->poolmetadatasize = 2 * DEFAULT_THIN_POOL_MAX_METADATA_SIZE;
+		} else if (lp->poolmetadatasize < (2 * DEFAULT_THIN_POOL_MIN_METADATA_SIZE)) {
+			if (arg_count(vg->cmd, poolmetadatasize_ARG))
+				log_warn("WARNING: Minimum supported pool metadata size is 2M.");
+			lp->poolmetadatasize = 2 * DEFAULT_THIN_POOL_MIN_METADATA_SIZE;
+		}
+
+		log_verbose("Setting pool metadata size to %" PRIu64 " sectors.",
+			    lp->poolmetadatasize);
+
+		if (!(lp->poolmetadataextents =
+		      extents_from_size(vg->cmd, lp->poolmetadatasize, vg->extent_size)))
+			return_0;
+	}
 
 	return 1;
 }
@@ -319,7 +335,7 @@ static int _read_size_params(struct lvcreate_params *lp,
 	}
 
 	if (arg_count(cmd, extents_ARG)) {
-		if (arg_sign_value(cmd, extents_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, extents_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative number of extents is invalid");
 			return 0;
 		}
@@ -329,7 +345,7 @@ static int _read_size_params(struct lvcreate_params *lp,
 
 	/* Size returned in kilobyte units; held in sectors */
 	if (arg_count(cmd, size_ARG)) {
-		if (arg_sign_value(cmd, size_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, size_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative size is invalid");
 			return 0;
 		}
@@ -346,7 +362,7 @@ static int _read_size_params(struct lvcreate_params *lp,
 			log_error("--poolmetadatasize may only be specified when allocating the thin pool.");
 			return 0;
 		}
-		if (arg_sign_value(cmd, poolmetadatasize_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, poolmetadatasize_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative poolmetadatasize is invalid.");
 			return 0;
 		}
@@ -359,7 +375,7 @@ static int _read_size_params(struct lvcreate_params *lp,
 			log_error("Virtual size in incompatible with thin_pool segment type.");
 			return 0;
 		}
-		if (arg_sign_value(cmd, virtualsize_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, virtualsize_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative virtual origin size is invalid");
 			return 0;
 		}
@@ -441,7 +457,7 @@ static int _read_mirror_params(struct lvcreate_params *lp,
 	lp->nosync = arg_is_set(cmd, nosync_ARG);
 
 	if (arg_count(cmd, regionsize_ARG)) {
-		if (arg_sign_value(cmd, regionsize_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, regionsize_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative regionsize is invalid");
 			return 0;
 		}
@@ -509,38 +525,12 @@ static int _read_raid_params(struct lvcreate_params *lp,
 	return 1;
 }
 
-static int _read_thin_params(struct lvcreate_params *lp,
-			     struct cmd_context *cmd)
-{
-	if (!seg_is_thin(lp)) {
-		if (lp->poolmetadatasize) {
-			log_error("Pool metadata size option is only for pool creation.");
-			return 0;
-		}
-		return 1;
-	}
-
-	if (lp->create_thin_pool) {
-		if (lp->poolmetadatasize > (2 * DEFAULT_THIN_POOL_MAX_METADATA_SIZE)) {
-			log_warn("WARNING: Maximum supported pool metadata size is 16GB.");
-			lp->poolmetadatasize = 2 * DEFAULT_THIN_POOL_MAX_METADATA_SIZE;
-		} else if (lp->poolmetadatasize < (2 * DEFAULT_THIN_POOL_MIN_METADATA_SIZE))
-			lp->poolmetadatasize = 2 * DEFAULT_THIN_POOL_MIN_METADATA_SIZE;
-		log_verbose("Setting pool metadata size to %" PRIu64 " sectors.",
-			    lp->poolmetadatasize);
-	} else if (lp->poolmetadatasize) {
-		log_error("Pool metadata size options is only for pool creation.");
-		return 0;
-	}
-
-	return 1;
-}
-
 static int _read_activation_params(struct lvcreate_params *lp, struct cmd_context *cmd)
 {
 	unsigned pagesize;
 
-	lp->activate = arg_uint_value(cmd, available_ARG, CHANGE_AY);
+	lp->activate = (activation_change_t)
+		arg_uint_value(cmd, available_ARG, CHANGE_AY);
 
 	if (lp->activate == CHANGE_AN || lp->activate == CHANGE_ALN) {
 		if (lp->zero && !seg_is_thin(lp)) {
@@ -649,10 +639,13 @@ static int _lvcreate_params(struct lvcreate_params *lp,
 	else
 		segtype_str = "striped";
 
-	lp->segtype = get_segtype_from_string(cmd, arg_str_value(cmd, type_ARG, segtype_str));
+	segtype_str = arg_str_value(cmd, type_ARG, segtype_str);
+
+	if (!(lp->segtype = get_segtype_from_string(cmd, segtype_str)))
+		return_0;
 
 	if (seg_unknown(lp)) {
-		log_error("Unable to create LV with unknown segment type %s.", arg_str_value(cmd, type_ARG, segtype_str));
+		log_error("Unable to create LV with unknown segment type %s.", segtype_str);
 		return 0;
 	}
 
@@ -686,7 +679,7 @@ static int _lvcreate_params(struct lvcreate_params *lp,
 			}
 			log_print("Redundant mirrors argument: default is 0");
 		}
-		if (arg_sign_value(cmd, mirrors_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, mirrors_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Mirrors argument may not be negative");
 			return 0;
 		}
@@ -731,8 +724,7 @@ static int _lvcreate_params(struct lvcreate_params *lp,
 	    !_read_size_params(lp, lcp, cmd) ||
 	    !get_stripe_params(cmd, &lp->stripes, &lp->stripe_size) ||
 	    !_read_mirror_params(lp, cmd) ||
-	    !_read_raid_params(lp, cmd) ||
-	    !_read_thin_params(lp, cmd))
+	    !_read_raid_params(lp, cmd))
 		return_0;
 
 	if (lp->snapshot && lp->thin && arg_count(cmd, chunksize_ARG))
@@ -741,7 +733,7 @@ static int _lvcreate_params(struct lvcreate_params *lp,
 		if (arg_count(cmd, chunksize_ARG))
 			log_warn("WARNING: Ignoring --chunksize when using an existing pool.");
 	} else if (lp->snapshot || lp->create_thin_pool) {
-		if (arg_sign_value(cmd, chunksize_ARG, 0) == SIGN_MINUS) {
+		if (arg_sign_value(cmd, chunksize_ARG, SIGN_NONE) == SIGN_MINUS) {
 			log_error("Negative chunk size is invalid");
 			return 0;
 		}
@@ -794,7 +786,7 @@ static int _lvcreate_params(struct lvcreate_params *lp,
 
 	lp->alloc = contiguous ? ALLOC_CONTIGUOUS : ALLOC_INHERIT;
 
-	lp->alloc = arg_uint_value(cmd, alloc_ARG, lp->alloc);
+	lp->alloc = (alloc_policy_t) arg_uint_value(cmd, alloc_ARG, lp->alloc);
 
 	if (contiguous && (lp->alloc != ALLOC_CONTIGUOUS)) {
 		log_error("Conflicting contiguous and alloc arguments");
